from typing import Optional

import requests
from urllib.parse import urlencode, urlparse

from ...configs.client_config import ClientConfig
from ...utils.unmeshed_common_utils import UnmeshedCommonUtils


class HttpRequestFactory:
    def __init__(self, client_config: ClientConfig):
        self.client_config = client_config
        self.base_url = client_config.get_base_url()
        self.port = client_config.get_port()
        self.bearer_value = f"Bearer client.sdk.{self.client_config.get_client_id()}.{UnmeshedCommonUtils.create_secure_hash(self.client_config.get_auth_token())}"

    def __build_uri(self, path: str, params: Optional[dict[str, str]] = None) -> str:
        if self.base_url.endswith("/"):
            self.base_url = self.base_url[:-1]

        parsed_url = urlparse(self.base_url)
        has_port = parsed_url.port is not None
        if has_port or self.base_url.startswith("https:"):
            url = f"{self.base_url}/{path}"
        else:
            url = f"{self.base_url}:{self.port}/{path}"
        if params:
            query_string = urlencode(params)
            return f"{url}?{query_string}"

        return url

    def create_get_request(self, path, params):
        uri = self.__build_uri(path, params)
        headers = {
            "Content-Type": "application/json",
            "Authorization": self.bearer_value
        }
        return requests.get(uri, headers=headers)

    def create_post_request(self, path, params, body):
        uri = self.__build_uri(path, params)
        headers = {
            "Content-Type": "application/json",
            "Authorization": self.bearer_value
        }
        return requests.post(uri, json=body, headers=headers)

    def create_put_request(self, path, params, body):
        uri = self.__build_uri(path, params)
        headers = {
            "Content-Type": "application/json",
            "Authorization": self.bearer_value
        }
        return requests.put(uri, json=body, headers=headers)

    def create_post_request_with_body(self, path, body):
        return self.create_post_request(path, None, body)