from dataclasses import dataclass, field
from typing import Any, Optional
from datetime import datetime

from .unmeshed_constants import UnmeshedConstants


@dataclass
class ProcessData:
    process_id: int
    process_type: 'UnmeshedConstants.ProcessType'
    trigger_type: 'UnmeshedConstants.ProcessTriggerType'
    namespace: str
    name: str
    status: 'UnmeshedConstants.ProcessStatus'

    version: int = 0
    history_id: str = None
    request_id: str = None
    correlation_id: str = None

    input: dict[str, Any] = field(default_factory=dict)
    output: dict[str, Any] = field(default_factory=dict)
    state: dict[str, Any] = field(default_factory=dict)
    secret_state: dict[str, Any] = field(default_factory=dict)
    auth_claims: dict[str, Any] = field(default_factory=dict)

    step_id_count: str = None
    shard_name: str = None
    shard_instance_id: str = None

    steps: list['StepId'] = field(default_factory=list)
    step_records: list['StepData'] = field(default_factory=list)

    created: datetime = None
    updated: datetime = None
    created_by: str = None

    def get_process_id(self) -> int:
        return self.process_id

    def set_process_id(self, process_id: int):
        self.process_id = process_id

    def get_process_type(self):
        return self.process_type

    def set_process_type(self, process_type):
        self.process_type = process_type

    def get_trigger_type(self):
        return self.trigger_type

    def set_trigger_type(self, trigger_type):
        self.trigger_type = trigger_type

    def get_namespace(self) -> str:
        return self.namespace

    def set_namespace(self, namespace: str):
        self.namespace = namespace

    def get_name(self) -> str:
        return self.name

    def set_name(self, name: str):
        self.name = name

    def get_version(self) -> int:
        return self.version

    def set_version(self, version: int):
        self.version = version

    def get_history_id(self) -> Optional[int]:
        return self.history_id

    def set_history_id(self, history_id: Optional[int]):
        self.history_id = history_id

    def get_request_id(self) -> Optional[str]:
        return self.request_id

    def set_request_id(self, request_id: Optional[str]):
        self.request_id = request_id

    def get_correlation_id(self) -> Optional[str]:
        return self.correlation_id

    def set_correlation_id(self, correlation_id: Optional[str]):
        self.correlation_id = correlation_id

    def get_status(self):
        return self.status

    def set_status(self, status):
        self.status = status

    def get_input(self) -> dict[str, Any]:
        return self.input

    def set_input(self, input_data: dict[str, Any]):
        self.input = input_data

    def get_output(self) -> dict[str, Any]:
        return self.output

    def set_output(self, output_data: dict[str, Any]):
        self.output = output_data

    def get_state(self) -> dict[str, Any]:
        return self.state

    def set_state(self, state_data: dict[str, Any]):
        self.state = state_data

    def get_secret_state(self) -> dict[str, Any]:
        return self.secret_state

    def set_secret_state(self, secret_state_data: dict[str, Any]):
        self.secret_state = secret_state_data

    def get_auth_claims(self) -> dict[str, Any]:
        return self.auth_claims

    def set_auth_claims(self, auth_claims_data: dict[str, Any]):
        self.auth_claims = auth_claims_data

    def get_step_id_count(self) -> int:
        return self.step_id_count

    def set_step_id_count(self, step_id_count: int):
        self.step_id_count = step_id_count

    def get_shard_name(self) -> Optional[str]:
        return self.shard_name

    def set_shard_name(self, shard_name: str):
        self.shard_name = shard_name

    def get_shard_instance_id(self) -> int:
        return self.shard_instance_id

    def set_shard_instance_id(self, shard_instance_id: int):
        self.shard_instance_id = shard_instance_id

    def get_steps(self) -> list['StepId']:
        return self.steps

    def set_steps(self, steps: list['StepId']):
        self.steps = steps

    def get_step_records(self) -> list['StepData']:
        return self.step_records

    def set_step_records(self, step_records: list['StepData']):
        self.step_records = step_records

    def get_created(self) -> datetime:
        return self.created

    def set_created(self, created: datetime):
        self.created = created

    def get_updated(self) -> datetime:
        return self.updated

    def set_updated(self, updated: datetime):
        self.updated = updated

    def get_created_by(self) -> str:
        return self.created_by

    def set_created_by(self, created_by: str):
        self.created_by = created_by
