from dataclasses import dataclass, field
from typing import Optional

from .unmeshed_constants import UnmeshedConstants


@dataclass
class StepQueueNameData:
    _org_id: Optional[int] = field(default=None)
    _namespace: Optional[str] = field(default=None)
    _step_type: UnmeshedConstants.StepType = field(default=UnmeshedConstants.StepType.WORKER)
    _name: str = field(default=None)

    def __init__(self, org_id: int, namespace: str, step_type: UnmeshedConstants.StepType, name: str):
        self.set_org_id(org_id)
        self.set_namespace(namespace)
        self.set_step_type(step_type)
        self.set_name(name)

    def get_org_id(self) -> int:
        return self._org_id

    def set_org_id(self, value: int) -> None:
        if value <= 0:
            raise ValueError("org_id must be a positive integer.")
        self._org_id = value

    def get_namespace(self) -> str:
        return self._namespace

    def set_namespace(self, value: str) -> None:
        if not value:
            raise ValueError("namespace cannot be empty.")
        self._namespace = value

    def get_step_type(self) -> UnmeshedConstants.StepType:
        return self._step_type

    def set_step_type(self, value: UnmeshedConstants.StepType) -> None:
        if not isinstance(value, UnmeshedConstants.StepType):
            raise ValueError("Invalid StepType value.")
        self._step_type = value

    def get_name(self) -> str:
        return self._name

    def set_name(self, value: str) -> None:
        if not value:
            raise ValueError("name cannot be empty.")
        self._name = value
